#include "pixelbuffer.hpp"

#define XPM_SKIP_LINES		3

static const PixelBuffer::Pixel colors[8]=
	{
	RGB(0,0,0),			//	 0 - Black
	RGB(255,0,0),		//	 1 - Light Red
	RGB(0,255,0),		//	 2 - Light Green
	RGB(255,255,0),		//	 3 - Yellow
	RGB(0,0,255),		//	 4 - Light Blue
	RGB(255,0,255),		//	 5 - Light Magenta
	RGB(0,255,255),		//	 6 - Light Cyan
	RGB(255,255,255),	//	 7 - High White
	};


void PixelBuffer::clear(Pixel pixel)
{
  int height = _height;
  int width = _width;
  int row = 0;
  for (;row!=height;++row) {
    int col = 0;
    for (;col!=width;++col) {
      writePixel(col,row,pixel);
    }
  }
}


struct FontGlyph {
  FontGlyph(FontInfo &f,int i)
  : font(f), index(i), glyph_data(0), col(0)
  {
    if (!font.isValidGlyph(index)) {
      return;
    }
    const int offset = index - font.glyph_start();
    int line = offset / glyphs_across;
    int row = line * font.char_height();
    int pos = offset % glyphs_across;
    col = pos * font.char_width();
    glyph_data = font.glyph_data()+XPM_SKIP_LINES+row;
  }
  
  bool isValid() const { return glyph_data!=0; }

  bool pixel(int x,int y) const
  {
    char c = glyph_data[y][col + x];
    if (c==' ') {
      return false;
    }
    return true;
  }

  static const int glyphs_across = 8;

  FontInfo &font;
  int index;
  const char **glyph_data;
  int col;
};

#if 0
#define GLYPHS_ACCROSS 8
static bool FontVal(FontInfo &font,int index,int x,int y)
{
  if (!font.isValidGlyph(index)) {
    return false;
  }
  const int offset = index - font.glyph_start();
  int line = offset / GLYPHS_ACCROSS;
  int row = line * font.char_height();
  int pos = offset % GLYPHS_ACCROSS;
  int col = pos * font.char_width();
  const char **glyph_data = font.glyph_data();
  char c = glyph_data[XPM_SKIP_LINES + row + y][col + x];
  if (c==' ') {
    return false;
  }
  return true;
}
#endif

void PixelBuffer::invertCharXY(int x1,int y1)
{
  int char_height = _font.char_height();
  int char_width = _font.char_width();

  int y = 0;
  for (;y!=char_height;++y) {
    int x = 0;
    for (;x!=char_width;++x) {
      Pixel color = readPixel(x+x1,y+y1);
      writePixel(x+x1,y+y1,inverseColor(color));
    }
  }
}

void PixelBuffer::invertChar(int row,int col)
{
  int char_height = _font.char_height();
  int char_width = _font.char_width();
  int x1 = col*char_width;
  int y1 = row*char_height;

  invertCharXY(x1,y1);
}
#if 0
void PixelBuffer::writeCharXY(int x1,int y1,char c)
{
  int char_height = _font.char_height();
  int char_width = _font.char_width();
  Pixel foreground = pixel(255,255,255);
  Pixel background = pixel(0,0,0);

#if 1
  FontGlyph glyph(_font,c);

  if (!glyph.isValid()) {
    int y = 0;
    for (;y < char_height;++y) {
      int x = 0;
      for (;x < char_width;++x) {
        writePixel(x+x1,y+y1,background);
      }
    }
    return;
  }

  int y = 0;
  for (;y < char_height;++y) {
    int x = 0;
    for (;x < char_width;++x) {
      Pixel color = glyph.pixel(x,y) ? foreground : background;
      writePixel(x+x1,y+y1,color);
    }
  }
#else
  int y = 0;
  for (;y < char_height;++y) {
    int x = 0;
    for (;x < char_width;++x) {
      Pixel color = FontVal(_font,c,x,y) ? foreground : background;
      writePixel(x+x1,y+y1,color);
    }
  }
#endif
}
#endif

void PixelBuffer::writeCharAbsoluteXY(int x1,int y1,char c, Pixel foreground, Pixel background)
{
  int char_height = _font.char_height();
  int char_width = _font.char_width();

  FontGlyph glyph(_font,c);

  if (!glyph.isValid()) {
    int y = 0;
    for (;y < char_height;++y) {
      int x = 0;
      for (;x < char_width;++x) {
        writePixelAbsolute(x+x1,y+y1,background);
      }
    }
    return;
  }

  int y = 0;
  for (;y < char_height;++y) {
    int x = 0;
    for (;x < char_width;++x) {
      Pixel color = glyph.pixel(x,y) ? foreground : background;
      writePixelAbsolute(x+x1,y+y1,color);
    }
  }
}
int PixelBuffer::writeStrAbsoluteXY(int x1,int y1,const char *str, Pixel foreground, Pixel background)
  {
  int char_width = _font.char_width();
  for(; *str; str++, x1 += char_width)
	  writeCharAbsoluteXY(x1, y1, *str, foreground, background);
  return x1;
  }
#if 0
void PixelBuffer::writeCharAbsoluteXY(int x1,int y1,char c)
{
  Pixel foreground = pixel(255,255,255);
  Pixel background = pixel(0,0,0);
  writeCharAbsoluteXY(x1, y1, c, foreground, background);
}

void PixelBuffer::writeChar(int row,int col,char c)
{
  int char_height = _font.char_height();
  int char_width = _font.char_width();
  int x1 = col*char_width;
  int y1 = row*char_height;
  writeCharXY(x1, y1, c);
}
#endif
void PixelBuffer::writeCharXY(int x1,int y1,CellBuffer &c)
{
  int char_height = _font.char_height();
  int char_width = _font.char_width();
  Pixel foreground = colors[c.fg_color & ansiColorMask];
  Pixel background = colors[c.bg_color & ansiColorMask];

  FontGlyph glyph(_font,c.ch);

  if (!glyph.isValid()) {
    int y = 0;
    for (;y < char_height;++y) {
      int x = 0;
      for (;x < char_width;++x) {
        writePixel(x+x1,y+y1,background);
      }
    }
    return;
  }

  int y = 0;
  for (;y < char_height;++y) {
    int x = 0;
    for (;x < char_width;++x) {
      Pixel color = glyph.pixel(x,y) ? foreground : background;
      writePixel(x+x1,y+y1,color);
    }
  }
}

void PixelBuffer::writeChar(int row,int col,CellBuffer &c)
{
  int char_height = _font.char_height();
  int char_width = _font.char_width();
  int x1 = col*char_width;
  int y1 = row*char_height;
  writeCharXY(x1, y1, c);
}
